;*********************************************************************
;
;   Filename:		LCD.ASM                               
;   Date:			27.07.2012        						
;   Author:			Ottmar  
;                              
;*********************************************************************
;	Initialisierung eines HD4480-basierenden LCD
;	fr 4bit-Interface
;	Hardware:	PIC16F628
;				Quarz 10 MHz
;**********************************************************************
;	Programmbeschreibung:
;	Ausgabe eines einfachen Textes im 2zeiligen LCD 
;**********************************************************************
;	PIC-Konfiguration
	LIST P=16F628
	#include <P16F628.INC>
	__CONFIG _CP_OFF & _LVP_OFF & _PWRTE_ON & _HS_OSC & _WDT_OFF & _MCLRE_ON 
	errorlevel -302	;"Register in operand not in bank 0. Ensure .."
	errorlevel -207	;"Found label after column 1."
;**********************************************************************
	CBLOCK	0x20		;Variables used for initializing the LCD
		d1,d2,d3		;Delay-Routinen
		Lcd_Data		;used for data transfer to LCD (ASCII-Code) 
		Lcd_Control		;used for HD44780 instructions		
		Lcd_Status		;to copy the LCD-PORT detecting busyflag
	ENDC
	;
	;LABELS 
	#define	LCD_PORT	PORTB	;Output-port LCD-data (e.g. Char.)
	#define	LCD_LAT		PORTB	;16F628 enthlt kein LATB-Register
								;daher wird PORTB zugewiesen	
	#define	LCD_TRIS	TRISB	;I/O direction register
	;
	RS		equ	0		;0/1 write data Data/Instruction to LCD		
	RW		equ	1		;0/1 write tot/read from LCd-Databus		
	E		equ	2		;1->0 enable H->L moves Dats into the LCD		
	BSYF	equ 7		;1/0 busyflag LCD not/redy to get datas/instructions
	;
	#define	Lcd_RS	LCD_PORT,RS	;Regist. Select	1/0 Data/Instruction 
	#define	Lcd_RW	LCD_PORT,RW	;Read/Write 	1/0 Lesen/Schreiben
	#define	Lcd_E	LCD_PORT,E	;Enable
;*********************************************************************
	ORG	0x0000		;RESET-Vector
	GOTO	INIT
;------------------------------------------------------------------
	ORG	0x0004		;INTERRUPT-Vector
	RETFIE			;not used
;*********************************************************************
INIT
	clrf	LCD_PORT	;initialize  PORT by setting the data latches
	banksel	LCD_TRIS	;bank with the data direction register
	movlw	b'00000000'	;value used to initalize LCD_TRISn
	movwf	LCD_TRIS	;sets B7:0 as outputs (for inputs use "1")
	;
	;PORTA is after Reset/StartUp initialized with ANALOG-inputs (comparator-mode!)
	clrf	PORTA		;initialize PORTA by setting the data latches
	banksel	TRISA		;bank with the data direction register
	movlw	b'0001111'	;turn comparators off, set RA4:3 as inputs		
	banksel	PORTA		;b5 is always "1", b7:6 depending on osc-mode
	;
	CALL	LCD_INIT	;initialize the HD44780-LCD
	;----------------------------------------------------------------
	GOTO	MAIN	
;*********************************************************************
MAIN
	;
	CALL	Lcd_Ready	;displays: "LCD ready 4 YOU"
	;
	GOTO	MAIN
;*********************************************************************
;		SUBROUTINES
;*********************************************************************
;
;
;*********************************************************************
	#include	<HD44780.INC>
;*********************************************************************
	END