;******************************************************************************
;	Filename:		LCD_INIT.INC
;	Date:			12.07.2012  
;	gendert		18.07.2012
;	Author:			Wil Kceg (Ottmar)
;	email:			wkceg@yahoo.com
;	Copyright:		free for non commercial use
;
;	Purpose;		Initialize a HD44780-LCD
;					This file is succesful tested with 
;					PIC16F1827, PIC16F628, PIC16F876
;******************************************************************************
	NOLIST
;VARIABLES in LCD_INIT.INC
;	CBLOCK	0x20				;Variables used for initializing the LCD
;		d1,d2,d3				;Delay-routines
;		Lcd_Data				;used for data transfer to LCD (ASCII-Code) 
;		Lcd_Control				;used for HD44780 instructions		
;		Lcd_Status				;used to copy the LCD-PORT, detecting busyflag
;	ENDC
;
;LABELS and CONSTANTS in LCD_INIT.INC
;	;						LCD_PORT Bit3:7	D4:D7 LCD-Data I/O (e.g.PORTB RB3:RB7)
;	;						LCD_PORT Bit3:7	D4:D7 LCD-Data I/O (e.g.PORTB RB3:RB7)
;	#define	LCD_PORT	PORTB	;(or other PORTx )
;	#define	LCD_LAT		LATB	;(or other LATx or PORT if LATx not exist)
;	#define	LCD_TRIS	TRISB	;(or other TRISx)
;
;	RS			equ	0			;0/1 write data Data/Instruction to LCD		
;	RW			equ	1			;0/1 write tot/read from LCd-Databus		
;	E			equ	2			;1->0 enable H->L moves Dats into the LCD
;	BSYF		equ 7			;1/0 busyflag LCD not/redy to get datas/instructions
;
;	#define		Lcd_RS	LCD_LAT,RS	;(e.g.PORT,RS) Regist. Select 1/0 Data/Instruction 
;	#define		Lcd_RW	LCD_LAT,RW	;(e.g.PORT,RW) 1/0 Read/Write 
;	#define		Lcd_E	LCD_LAT,E	;(e.g.PORT,E)  Enable toggling: 0-1-0
;------------------------------------------------------------------------------
; Use only one of this constants for tagging your LCD!
;	LCD_TYPE	equ	b'00000001'	;LCD 1 line 
;	LCD_TYPE	equ	b'00000011'	;LCD 2 lines 
;	LCD_TYPE	equ	b'00001111'	;LCD 4 lines 
;	Don't use other Bits than 3:0!
;
;SUBROUTINES in LCD_INIT.INC
;	LCD_INIT		Main-routine
;	Control8Bit		Initializing Interface as 8Bit-Bus
;	OutLcdControl	sends a command (e.g. adress, function) via 4bit-interface
;	OutLcdData		sends a ansii-character via 4bit-interface
;	LcdBusy			waiting LCD ready to get the next command/data?
;	Delay1ms		
;	Lcd_Ready		LCD selftest displays "LCD ready 4 you!"
;------------------------------------------------------------------------------
;LCD-Remarks
;	When RS = 0 and RW = 1, the busy flag is output to DB7. The next 
;	instruction must be written after ensuring that the busy flag is 0.
;
;	RS	R/W	Operation
;	0	0	IR write as an internal operation (display clear, etc.)
;	0	1	Read busy flag (DB7) and address counter (DB0 to DB6)
;	1	0	DR write as an internal operation (DR to DDRAM or CGRAM)
;	1	1	DR read as an internal operation (DDRAM or CGRAM to DR)
;******************************************************************************
	LIST
LCD_INIT						;Quelle: www.sprut.de
	movlw	D'255'				;250 ms Pause nach dem Einschalten
	CALL	Delay1ms		
	;
	BANKSEL	LCD_LAT				;changes to bank with LATB (bank 2)
	movlw	B'00110000'			;1. Function set
	movwf	LCD_LAT				;	(Interface is 8 bits long.)
	bsf		Lcd_E				;	Toggeln LCD_LAT,E 
	nop	
	bcf		Lcd_E
	BANKSEL	LCD_PORT			;;changes to bank with PORTB (bank 0)
	movlw	D'50'				;	50 ms Pause
	CALL	Delay1ms
	movlw	B'00110000'			;2 Function set
	CALL	Control8Bit			;	(Interface is 8 bits long.)
	;
	movlw	B'00110000'			;3. Function set
	CALL 	Control8Bit			;	(Interface is 8 bits long.)
	;
	movlw	B'00110000'			;4. Function set
	CALL 	Control8Bit			;	(Interface is 4 bits long. 
	;
	movlw	B'00000001'			;	Display lschen und cusor home
	CALL	OutLcdControl
	;
	movlw	B'00101000'			;5. Function set 2 and more lines 5x8 Dot 
;	movlw	b'00100000'			;	Function set LCD single Line 5x8 Dot 
	CALL	OutLcdControl		;	send function to LCD		
	;
	movlw	B'00001000'			;6. display off
	CALL	OutLcdControl
	;
	movlw	B'00000110'			;7. entry mode, increment
	CALL	OutLcdControl		;	disable display-shift
	;
	movlw	B'00000011'			;8. cursor home, cursor home
	CALL	OutLcdControl
	;
	movlw	B'00001100'			;9. Set	Display ON, Cursor OFF Blinken OFF
								;	Bit3	Bit2	Bit1		Bit0
	CALL	OutLcdControl
	BANKSEL	LCD_PORT			;back to Bank 0
	RETURN
;---------------------------------------------------------------------------
; ein Steuerbyte 8-bittig bertragen
Control8Bit
	BANKSEL	LCD_LAT				;changes to bank with LATB
	movwf	LCD_LAT				;LCD_PORT/Latch send Control to LCD
	bsf		Lcd_E				;LCD_PORT,2	toggle
	nop
	bcf		Lcd_E				;LCD_PORT,2	toggle
	movlw	d'10'				;
	CALL	Delay1ms
	RETURN
;---------------------------------------------------------------------------
LcdBusy		; Checks the LCD to be ready to receive a instruction
	BANKSEL LCD_TRIS			;changes to bank with TRISB
	movlw	B'11110000'			;Bitmask 1->Inputs =->Outputs
	iorwf   LCD_TRIS,f			;TRISB,B7:4 set I/O as Inputs	
BusyLoop		
	BANKSEL	LCD_LAT				;changes to bank with LATB
	bcf		Lcd_RS				;LCD_PORT,0	set mode "LCD ready for instructions"	
	bsf		Lcd_RW				;LCD_PORT,1 set mode "read form LCD-Port"
	bsf		Lcd_E				;LCD_PORT,0 Enable selected modes
	nop							;1 cycle delay
	BANKSEL	LCD_PORT			;Wechsel zu Bank 2
	movf	LCD_PORT,w			;copy 4bit-bus levels to WREG 
	movwf	Lcd_Status			;"    WREG to storage-variable
	BANKSEL	LCD_LAT				;Wechsel zu Bank 2
	bcf		Lcd_E				;disable selectet modes
	nop
	bsf		Lcd_E				;Enable
	nop
	bcf		Lcd_E
	#ifdef __DEBUG				; in MPLAB-DEBUG-Mode busy-flag not exist
	#else
		BANKSEL	LCD_PORT
		btfsc 	Lcd_Status,BSYF	;LCD busy? Lcd_Status,7
		goto  	BusyLoop		;YES try it again
	#endif
	bcf		Lcd_RW				;
	BANKSEL LCD_TRIS			;changes to bank with TRISB
	movlw	B'00001111'			;Bitmask 1->Inputs =->Outputs
	andwf   LCD_TRIS, f			;TRISB,B7:4 set I/O as Outputs  
	BANKSEL LCD_PORT		;back to origin bank
	RETURN
;---------------------------------------------------------------------------
OutLcdControl	;sends 1 instruction-byte to the LCD 4bit-control-bus
	banksel	LCD_PORT
	movwf	Lcd_Control
	call	LcdBusy				;check LCD ready to receive next instruction
	movf	Lcd_Control, w		;contents instruction (e.g. LCD-adress)
	ANDLW	b'11110000'			;Bitmask to select HI-nibble 
	BANKSEL	LCD_LAT				;Wechsel zu Bank 2
	movwf	LCD_LAT				;LCD_PORT write Nibble to 4bit-control-bus
	bsf		Lcd_E				;toggle
	nop
	bcf		Lcd_E				;toggle
	banksel	LCD_PORT
	SWAPF	Lcd_Control, w		;change the nibbles
	ANDLW	b'11110000'			;Bitmask to select the LO-nibble 
	BANKSEL	LCD_LAT
	movwf	LCD_LAT				;LCD_PORT/LAT write Nibble to 4bit-bus
	bsf		Lcd_E				;toggle
	nop
	bcf		Lcd_E				;Control-bus is now disabled
	BANKSEL	LCD_PORT			;bank 0
	RETURN
;---------------------------------------------------------------------------
OutLcdData		;sends 1 data-byte do LCD 4bit-bus
	BANKSEL	LCD_PORT
	movwf	Lcd_Data
	CALL	LcdBusy				;check LCD ready to receive next instruction
	movf	Lcd_Data,w			;contents LCD-Data (Charakter)
	ANDLW	b'11110000'			;Bitmask to select HI-nibble
	BANKSEL	LCD_LAT				;Wechsel zu Bank 2
	movwf	LCD_LAT					;LCD_PORT write Nibble to 4bit-databus
	bsf		Lcd_RS				;1=LCD-Data-mode
	bsf		Lcd_E				;enable LCD 4bit-databus
	nop							;toggle
	bcf		Lcd_E				;disable databus
	BANKSEL	LCD_PORT	
	SWAPF	Lcd_Data, w			;change the nibbles
	ANDLW	b'11110000'			;Bitmask to select the LO-nibble 
	BANKSEL	LCD_LAT
	movwf	LCD_LAT						;LCD_PORT write Nibble to 4bit-control-bus
	bsf		Lcd_RS				;1=LCD-Data-mode
	bsf		Lcd_E				;enable LCD 4bit-databus
	nop							;toggle
	bcf		Lcd_E				;databus is now disabled	
	bcf		Lcd_RS				;=0 LCD-control-mode
	BANKSEL	LCD_PORT			;bank 0
	RETURN
;------------------------------------------------------------------------------
Lcd_Ready					;executiontime 935s
	movlw	b'10000000'		;function set DDRAM-adress=0
	CALL	OutLcdControl	;send instruction to LCD
	movlw	"L"				;copy character as ansii-code to WREG
	CALL	OutLcdData		;send charactercode to LCD		
	movlw	"C"
	CALL	OutLcdData
	movlw	"D"
	CALL	OutLcdData
	movlw	" "
	CALL	OutLcdData
	movlw	"r"
	CALL	OutLcdData
	movlw	"e"
	CALL	OutLcdData
	movlw	"a"
	CALL	OutLcdData
	movlw	"d"
	CALL	OutLcdData
	;
	movlw	40h				;LCD 2x8: DDRAM-Adress line 2 				
	iorlw	b'10000000'		;function set + DDRAM adress
	CALL	OutLcdControl	;send instruction to LCD
	;
	movlw	"y"
	CALL	OutLcdData
	movlw	" "
	CALL	OutLcdData
	movlw	"4"
	CALL	OutLcdData
	movlw	" "
	CALL	OutLcdData
	movlw	"Y"
	CALL	OutLcdData
	movlw	"O"
	CALL	OutLcdData
	movlw	"U"
	CALL	OutLcdData
	movlw	"!"
	CALL	OutLcdData
	RETURN
;*********************************************************************
Delay1ms		;Actual delay = 0.001 seconds, Clock frequency = 4 MHz
	movwf	d3	;Transfer-Variable: Milliseconds (1-255) in the WREG.
Delay_1ms_0
	movlw	0xC6
	movwf	d1
	movlw	0x01
	movwf	d2
Delay_1ms_1
	decfsz	d1, f
	goto	$+2
	decfsz	d2, f
	goto	Delay_1ms_1
	goto	$+1
	decfsz	d3,f
	GOTO 	Delay_1ms_0
	RETURN
;*********************************************************************
